@@ -36,6 +36,11 @@ import (
 	"github.com/prometheus/pushgateway/storage"
 )
 
+const (
+	pushMetricName = "push_time_seconds"
+	pushMetricHelp = "Last Unix time when this group was changed in the Pushgateway."
+)
+
 // Push returns an http.Handler which accepts samples over HTTP and stores them
 // in the MetricStore. If replace is true, all metrics for the job and instance
 // given by the request are deleted before new ones are stored.
@@ -96,13 +101,19 @@ func Push(
 				metricFamilies, err = parser.TextToMetricFamilies(r.Body)
 			}
 			if err != nil {
-				http.Error(w, err.Error(), http.StatusInternalServerError)
+				http.Error(w, err.Error(), http.StatusBadRequest)
 				return
 			}
+			if timestampsPresent(metricFamilies) {
+				http.Error(w, "pushed metrics must not have timestamps", http.StatusBadRequest)
+				return
+			}
+			now := time.Now()
+			addPushTimestamp(metricFamilies, now)
 			sanitizeLabels(metricFamilies, labels)
 			ms.SubmitWriteRequest(storage.WriteRequest{
 				Labels:         labels,
-				Timestamp:      time.Now(),
+				Timestamp:      now,
 				MetricFamilies: metricFamilies,
 			})
 			w.WriteHeader(http.StatusAccepted)
@@ -183,10 +194,16 @@ func LegacyPush(
 				http.Error(w, err.Error(), http.StatusInternalServerError)
 				return
 			}
+			if timestampsPresent(metricFamilies) {
+				http.Error(w, "pushed metrics must not have timestamps", http.StatusBadRequest)
+				return
+			}
+			now := time.Now()
+			addPushTimestamp(metricFamilies, now)
 			sanitizeLabels(metricFamilies, labels)
 			ms.SubmitWriteRequest(storage.WriteRequest{
 				Labels:         labels,
-				Timestamp:      time.Now(),
+				Timestamp:      now,
 				MetricFamilies: metricFamilies,
 			})
 			w.WriteHeader(http.StatusAccepted)
@@ -275,3 +292,31 @@ func splitLabels(labels string) (map[string]string, error) {
 	}
 	return result, nil
 }
+
+// Checks if any timestamps have been specified.
+func timestampsPresent(metricFamilies map[string]*dto.MetricFamily) bool {
+	for _, mf := range metricFamilies {
+		for _, m := range mf.GetMetric() {
+			if m.TimestampMs != nil {
+				return true
+			}
+		}
+	}
+	return false
+}
+
+// Add metric to indicate the push time.
+func addPushTimestamp(metricFamilies map[string]*dto.MetricFamily, t time.Time) {
+	metricFamilies[pushMetricName] = &dto.MetricFamily{
+		Name: proto.String(pushMetricName),
+		Help: proto.String(pushMetricHelp),
+		Type: dto.MetricType_GAUGE.Enum(),
+		Metric: []*dto.Metric{
+			{
+				Gauge: &dto.Gauge{
+					Value: proto.Float64(float64(t.UnixNano()) / 1e9),
+				},
+			},
+		},
+	}
+}
